import { Token } from './Token';
export function encode(tokens) {
    if (tokens instanceof Token) {
        return new Token(tokens.type, encode(tokens.content), tokens.alias);
    }
    else if (Array.isArray(tokens)) {
        return tokens.map(encode);
    }
    return tokens
        .replace(/&/g, '&amp;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;')
        .replace(/\u00a0/g, ' ');
}
export function clone(value) {
    const stringValue = String(value);
    if (stringValue === '[object Map]') {
        const temp = new Map();
        for (let [key, val] of value) {
            temp.set(key, clone(val));
        }
        return temp;
    }
    else if (stringValue === '[object Object]') {
        return Object.keys(value).reduce((p, c) => Object.assign(p, { [c]: clone(value[c]) }), {});
    }
    else if (Array.isArray(value)) {
        return value.map(clone);
    }
    return value;
}
export function insertBefore(parent, insertBeforeKey, def) {
    const temp = new Map();
    let removeKeys = false;
    const entries = parent.entries();
    for (const [key, value] of entries) {
        if (key === insertBeforeKey) {
            removeKeys = true;
        }
        if (removeKeys) {
            parent.delete(key);
            temp.set(key, value);
        }
    }
    for (const [key, value] of def) {
        parent.set(key, value);
    }
    for (const [key, value] of temp) {
        parent.set(key, value);
    }
    return parent;
}
export function setIn(def, path, value) {
    let val = def;
    let i = 0;
    while (i < path.length - 1) {
        if (typeof value === 'undefined' || value === null) {
            throw new Error(`Cannot resolve the path [${path.join(', ')}]`);
        }
        const curr = path[i];
        val = val instanceof Map ? val.get(curr) : val[curr];
        i++;
    }
    if (val instanceof Map) {
        val.set(path[i], value);
    }
    else {
        val[path[i]] = value;
    }
}
export function getIn(def, path) {
    let val = def;
    let i = 0;
    while (i < path.length) {
        const curr = path[i];
        val = val instanceof Map ? val.get(curr) : val[curr];
        i++;
    }
    return val;
}
