"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const hooks_1 = require("./hooks");
const tokenTypes = {
    comment(env) {
        env.attributes.spellcheck = 'true';
    },
    keyword(env) {
        env.classes.push(`keyword-${env.content.toLowerCase().trim()}`);
    },
    punctuation(env) {
        switch (env.content) {
            case '(':
            case ')':
                env.classes.push('brackets-parentheses');
                return;
            case '[':
            case ']':
                env.classes.push('brackets-square');
                return;
            case '<':
            case '>':
                env.classes.push('brackets-angle');
                return;
            case '{':
            case '}':
                env.classes.push('brackets-braces');
                return;
        }
    }
};
class Token {
    constructor(type, content, alias) {
        this.type = type;
        this.content = content;
        this.alias = alias;
    }
    static stringify(token, language, parent) {
        if (typeof token === 'string') {
            return token;
        }
        if (Array.isArray(token)) {
            return token.map(element => Token.stringify(element, language, token)).join('');
        }
        const env = {
            type: token.type,
            content: Token.stringify(token.content, language, parent),
            tag: 'span',
            classes: ['token', token.type],
            attributes: {},
            language,
            parent
        };
        if (env.type in tokenTypes)
            tokenTypes[env.type](env);
        if (token.alias) {
            const aliases = Array.isArray(token.alias) ? token.alias : [token.alias];
            env.classes.push(...aliases);
        }
        hooks_1.run('wrap', env);
        const attributes = Object.keys(env.attributes).reduce((prev, key) => `${prev}${key}="${env.attributes[key]}" `, '');
        return `<${env.tag} class="${env.classes.join(' ')}" ${attributes}>${env.content}</${env.tag}>`;
    }
}
exports.Token = Token;
