"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const clike_1 = require("./clike");
const utils_1 = require("../utils");
let ruby = utils_1.clone(clike_1.clike);
exports.ruby = ruby;
ruby.set('comment', [/#(?!\{[^\r\n]*?\}).*/, /^=begin(?:\r?\n|\r)(?:.*(?:\r?\n|\r))*?=end/m]);
ruby.set('keyword', /\b(?:alias|and|BEGIN|begin|break|case|class|def|define_method|defined|do|each|else|elsif|END|end|ensure|false|for|if|in|module|new|next|nil|not|or|raise|redo|require|rescue|retry|return|self|super|then|throw|true|undef|unless|until|when|while|yield)\b/);
const rest = utils_1.clone(ruby);
const stringInterpolation = new Map([
    [
        'interpolation',
        {
            pattern: /#\{[^}]+\}/,
            inside: new Map([
                [
                    'delimiter',
                    {
                        pattern: /^#\{|\}$/,
                        alias: 'tag'
                    }
                ],
                ['rest', rest]
            ])
        }
    ]
]);
utils_1.insertBefore(ruby, 'keyword', new Map([
    [
        'regex',
        [
            {
                pattern: /%r([^a-zA-Z0-9\s{(\[<])(?:(?!\1)[^\\]|\\[\s\S])*\1[gim]{0,3}/,
                inside: stringInterpolation,
                greedy: true
            },
            {
                pattern: /%r\((?:[^()\\]|\\[\s\S])*\)[gim]{0,3}/,
                inside: stringInterpolation,
                greedy: true
            },
            {
                // Here we need to specifically allow interpolation
                pattern: /%r\{(?:[^#{}\\]|#(?:\{[^}]+\})?|\\[\s\S])*\}[gim]{0,3}/,
                inside: stringInterpolation,
                greedy: true
            },
            {
                pattern: /%r\[(?:[^\[\]\\]|\\[\s\S])*\][gim]{0,3}/,
                inside: stringInterpolation,
                greedy: true
            },
            {
                pattern: /%r<(?:[^<>\\]|\\[\s\S])*>[gim]{0,3}/,
                inside: stringInterpolation,
                greedy: true
            },
            {
                pattern: /(^|[^/])\/(?!\/)(\[.+?]|\\.|[^/\\\r\n])+\/[gim]{0,3}(?=\s*($|[\r\n,.;})]))/,
                lookbehind: true,
                greedy: true
            }
        ]
    ],
    ['variable', /[@$]+[a-zA-Z_]\w*(?:[?!]|\b)/],
    ['symbol', /:[a-zA-Z_]\w*(?:[?!]|\b)/]
]));
utils_1.insertBefore(ruby, 'number', new Map([
    [
        'builtin',
        /\b(?:Array|Bignum|Binding|Class|Continuation|Dir|Exception|FalseClass|File|Stat|Fixnum|Float|Hash|Integer|IO|MatchData|Method|Module|NilClass|Numeric|Object|Proc|Range|Regexp|String|Struct|TMS|Symbol|ThreadGroup|Thread|Time|TrueClass)\b/
    ],
    ['constant', /\b[A-Z]\w*(?:[?!]|\b)/]
]));
ruby.set('string', [
    {
        pattern: /%[qQiIwWxs]?([^a-zA-Z0-9\s{(\[<])(?:(?!\1)[^\\]|\\[\s\S])*\1/,
        greedy: true,
        inside: stringInterpolation
    },
    {
        pattern: /%[qQiIwWxs]?\((?:[^()\\]|\\[\s\S])*\)/,
        greedy: true,
        inside: stringInterpolation
    },
    {
        // Here we need to specifically allow interpolation
        pattern: /%[qQiIwWxs]?\{(?:[^#{}\\]|#(?:\{[^}]+\})?|\\[\s\S])*\}/,
        greedy: true,
        inside: stringInterpolation
    },
    {
        pattern: /%[qQiIwWxs]?\[(?:[^\[\]\\]|\\[\s\S])*\]/,
        greedy: true,
        inside: stringInterpolation
    },
    {
        pattern: /%[qQiIwWxs]?<(?:[^<>\\]|\\[\s\S])*>/,
        greedy: true,
        inside: stringInterpolation
    },
    {
        pattern: /("|')(?:#\{[^}]+\}|\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,
        greedy: true,
        inside: stringInterpolation
    }
]);
